/**
 * @file FootyView.h
 * @brief Footỹr[ǗsCFootyViewNX̐錾
 * @author Shinji Watanabe
 * @version 1.0
 */

#pragma once

#include "FontObjects.h"
#include "FootyStatus.h"
#include "DoubleBuffering.h"
#include "FootyDoc.h"
#include "Caret.h"

#define LINEWIDTH_DEFAULT	50
#define RULERHEIGHT_DEFAULT	10

/*bool extern DrawAllUserFuncs;					//!< Ssĕ` by Tetr@pod
bool extern IsUFUpdateTiming;					//!< XVׂH by Tetr@pod*/

class CFootyView
{
//publico֐//////////////////////////////////////////////////////////
public:
	CFootyView();
	virtual ~CFootyView();

	// r[̐
	void Initialize(CFontObjects *pFonts,CFootyView *pView,CFootyDoc *pDocument,CFootyStatus *pStatus,int nFootyID,int nViewID);
	bool CreateFootyView( HWND hWndParent, HINSTANCE hInstance );
	void DestroyFootyView();
	
	// rbg}bv̓ǂݍ
	static void LoadLineIcons(HINSTANCE hInstance);
	static void ReleaseLineIcons();
	
	// 擾
	inline HWND GetWnd(){return m_hWnd;}
	inline bool IsFocused() const {return m_bIsFocused;}
	inline int GetRulerHeight() const {return m_nRulerHeight;}
	inline int GetLineCountWidth() const {return m_nLineCountWidth;}
	inline size_t GetVisibleLines() const {return m_nVisibleLines;}
	inline size_t GetVisibleColumns() const {return m_nVisibleColumns;}
	inline void SetShiftLock(bool bLocked){m_bShiftLocked = bLocked;}
	inline bool IsShiftLocked() const {return m_bShiftLocked;}
	inline int GetMarkVisible() const {return m_nMarkVisible;}
	inline int GetUnderlineVisible(){return m_bUnderlineVisible ? 1 : 0;}
	
	// GUI֌W̏
	void SetVisible(bool bVisible);
	inline bool SetVisibleCols()
	{
		return MoveWin(m_x,m_y,m_nWidth,m_nHeight,false);
	}
	void SetLineWidth(int nWidth);
	void SetRulerHeight(int nHeight);
	void SetMarginHeight(int nHeight);
	bool MoveWin(int x,int y,int nWidth,int nHeight,bool bRefresh = true);
	bool Refresh(bool bSendOther = true);
	void SetFocus(){::SetFocus(m_hWnd);}
	bool Refresh(HDC hDC,const RECT *rc);
	void CaretMove();
	void SetMarkVisible(int nMark){m_nMarkVisible = nMark;}
	void SetUnderlineVisible(int nUnderline){m_bUnderlineVisible = nUnderline ? true : false;}
	void SetUnderlineDraw(bool flag);
	
	/*Lbgʒu\ʒu𒲐鏈*/
	inline void SetFirstColumn(int nFirstColumn){m_nFirstVisibleColumn = nFirstColumn;}
	bool AdjustVisiblePos();
	bool AdjustVisibleLine();

	bool RenderLines2();

//privateo֐/////////////////////////////////////////////////////////
private:
	//! Ï`̂߂̏
	struct SelOldInfo
	{
		HPEN m_hPen,m_hPenOld;			//!< y
		HBRUSH m_hBrush,m_hBrushOld;	//!< uV
		int m_nRopOld;					//!< ROP[h
	};

	static LRESULT CALLBACK FootyWinProc(HWND hWnd,UINT msg,WPARAM wParam,LPARAM lParam);
	LRESULT MainProc(UINT msg,WPARAM wParam,LPARAM lParam);
#ifndef UNDER_CE
	static unsigned int GetWheelMsg();
#endif	/*not defined UNDER_CE*/
	
	// Ot@CJ
	void StartApplication(const wchar_t *szFileName);
	
	// o[̏Ԃݒ肷
	void SetBars();

	// L[{[h֘ÃbZ[W
	void OnImeComposition(LPARAM lParam);
	LRESULT OnImeRequest(WPARAM wParam,LPARAM lParam);
	void OnChar(wchar_t wc);
	bool OnKeyDown(int nKeyCode);
	
	// tH[JX֘ÃbZ[W
	void OnGotFocus();
	void OnLostFocus();
	
	// }EX֘ÃbZ[W
	void OnMouseDown(int x, int y);
	void OnMouseMove(int x, int y, bool bFromScrollTimer = false);
	void OnMouseUp(int x, int y);
	void OnMouseWheel(bool bIsForward);
	void OnMouseDouble(int x,int y);

	// XN[֌W̏
	void OnAutoVerticalTimer();
	void OnAutoHorizontalTimer();
	void StartVerticalScrollTimer( int nScrollNum );
	void StartHorizontalScrollTimer( int nScrollNum );
	void StopVerticalScrollTimer();
	void StopHorizontalScrollTimer();
	void OnVScroll( WPARAM wParam );
	void OnHScroll( WPARAM wParam );
	
	// vZñ[`
	bool CalcInfoFromMouse(int x,int y,CEditPosition *pPos);
	bool CalcLineCount(int y,CEditPosition *pStart,CEditPosition *pEnd);
	enum RetOnUrl
	{
		ONURL_NONE,				//!< ɂȂ
		ONURL_URL,				//!< URL
		ONURL_MAIL,				//!< [
		ONURL_LABEL,			//!< x by Tetr@pod
		ONURL_FUNC,				//!< ߁E֐ by Tetr@pod
	};
	RetOnUrl IsOnUrl(CEditPosition *pPos,CUrlInfo *pInfo);
	
	// _O֘Ȁ
	bool CaretRefresh();
	bool ScrollRefresh();
	bool LineChangedRefresh(LinePt pLine1, LinePt pLine2);
	inline bool LineChangedRefresh()
	{
		return LineChangedRefresh
		(
			m_pDocuments->GetCaretPosition()->GetLinePointer(),
			m_pDocuments->GetCaretPosition()->GetLinePointer()
		);
	}

	// _Op[`
	bool RenderLines(LinePt pStartLine,LinePt pEndLine);
	inline int GetTextPosX(int nColumn){return m_nLineCountWidth + 
		(nColumn - m_nFirstVisibleColumn) * (m_pFonts->GetWidth() + m_nWidthMargin);}
	static void DrawRectangle(HDC hDC,int x1,int y1,int x2,int y2,COLORREF color);
	static void DrawLine(HDC hDC,int x1,int y1,int x2,int y2,COLORREF color,int penType = PS_SOLID);
	bool RendLine(HDC hDC,LinePt pLine,int y,bool bEnd);
	bool InitDcForSelect(HDC hDC,SelOldInfo *pSelInfo);
	bool ReleaseDcForSelect(HDC hDC,SelOldInfo *pSelInfo);
	bool RendSelected(HDC hDC,CEthicLine *pLine,int y);
	bool RendLineCount(HDC hDC,const RECT *rc);
	bool RendBorderLine(HDC hDC,const RECT *rc);
	bool RendRuler(HDC hDC,const RECT *rc);
	bool RendUnderLine(HDC hDC,const RECT *rc);

//publicoϐ//////////////////////////////////////////////////////////
public:
	// }[W
	int m_nWidthMargin;							//!< ̃̕}[W
	int m_nHeightMargin;						//!< ̍}[W
	
	// ֐|C^
	Footy2FuncFocus m_pFuncFocus;				//!< tH[JX擾Cxg
	void *m_pDataFocus;							//!< ̃f[^

//privateoϐ/////////////////////////////////////////////////////////
private:
	//! }EXhbO
	enum enDragMode
	{
		DRAGMODE_NONE,							//!< hbOȂ
		DRAGMODE_MOUSE,							//!< }EXhbO
		DRAGMODE_LINE,							//!< sԍ\̈
		DRAGMODE_OLE,							//!< OLE DnD
	};
	
	//! ^C}[ID
	typedef enum
	{
		eTimer_AutoVertical = 1,				//!< XN[^C}[
		eTimer_AutoHorizontal,					//!< XN[^C}[
	} timer_t;

	//! 萔l
	enum enFixedNumber
	{
		WHEEL_MOUSE = 3,						//!< zC[}EXŃXN[s
		SCROLL_TIME = 150,						//!< XN[^C}[̊Ԋu(ms)
		PIXEL_PER_LINESPEED = 50,				//!< XN[̎́A񓖂艽XN[邩̃sNZ
		PIXEL_PER_COLUMNSPEED = 20,				//!< XN[̎́A񓖂艽XN[邩̃sNZ
		
		NUM_LINEICONS = sizeof(int) * 8,		//!< sԍACR
		LINEICON_MARGIN = 1,					//!< sԍACR̃}[W
		ICON_HEIGHT = 16,						//!< sԍACR̍
		ICON_WIDTH = 16,						//!< sԍACR̕
	};

	// EBhE
	int m_x,m_y;								//!< EBhËʒu
	int m_nWidth,m_nHeight;						//!< EBhȆ傫
	HWND m_hWnd;								//!< EBhEnh
	HWND m_hOwner;								//!< I[i[nh
	int m_nRop2;								//!< X^Iy[Vԍ
	bool m_bIsFocused;							//!< tH[JXĂ
	int m_nCaretWidth;							//!< Lbg̕
	bool m_bVisible;							//!< \Ă邩H
	CDoubleBuffering m_cDoubleBuffering;		//!< _uobt@O
	static HBITMAP m_bmpIcons[ NUM_LINEICONS ];	//!< sACR

	// VXe
	int m_nFootyID;								//!< FootyID
	int m_nViewID;								//!< r[ID
	CFontObjects *m_pFonts;						//!< tHg
	CFootyDoc *m_pDocuments;					//!< hLg
	CFootyStatus *m_pStatus;					//!< Xe[^X|C^
	CFootyView *m_pView;						//!< ̃r[
	bool m_bAutoIndentMode;						//!< Cfgsǂ

	// XN[f[^
	size_t m_nVisibleLines;						//!< \\ȍs
	size_t m_nVisibleColumns;					//!< \\Ȍ
	SCROLLINFO m_vInfo;							//!< XN[o[̏
	SCROLLINFO m_hInfo;							//!< XN[o[̏
#ifndef UNDER_CE
	static unsigned int  m_uWheelMsg;			//!< }EXzC[̃bZ[W
#endif	/*not defined UNDER_CE*/
	bool m_bAutoScrollVertical;					//!< XN[̍Œ
	bool m_bAutoScrollHorizontal;				//!< XN[̍Œ
	int m_nAutoScrollNumVertical;				//!< XN[̗
	int m_nAutoScrollNumHorizontal;				//!< XN[̗

	// e傫̏
	int m_nRulerHeight;							//!< [[
	int m_nLineCountWidth;						//!< sԍ\̈敝
	int m_nLineCountMargin;						//!< sԍ\̈̃}[W
	
	// \֘Ȁ
	CCaret m_cCaret;							//!< Lbg
	int m_nFirstVisibleColumn;					//!< ŏɕ\Ă錅
	int m_nNowMousePosX,m_nNowMousePosY;
	int m_nMarkVisible;							//!< GfB^̕`
	bool m_bUnderlineVisible;					//!< s̕`
	bool m_bUnderlineDraw;						// tH[JX̃A_[C̕`
	
	// ꎞIȕϐ
	unsigned long m_nIgnoreKey;					//!< L[
	bool m_bShiftLocked;						//!< VtgbNĂ邩ǂ
	enDragMode m_nDragMode;						//!< hbO
};

/*[EOF]*/
